@@ -21,7 +21,7 @@
 
 #define FUSE_USE_VERSION 27
 #define PROGRAM "fuse-zip"
-#define VERSION "0.2.11"
+#define VERSION "0.2.12"
 #define ERROR_STR_BUF_LEN 0x100
 #define STANDARD_BLOCK_SIZE (512)
 
@@ -34,7 +34,6 @@
 #include <unistd.h>
 #include <limits.h>
 #include <syslog.h>
-#include <sys/xattr.h>
 #include <sys/types.h>
 #include <sys/statvfs.h>
 
@@ -241,7 +240,7 @@ static int fusezip_release (const char *path, struct fuse_file_info *fi) {
 static int fusezip_ftruncate(const char *path, off_t offset, struct fuse_file_info *fi) {
     (void) path;
 
-    return ((FileNode*)fi->fh)->truncate(offset);
+    return -((FileNode*)fi->fh)->truncate(offset);
 }
 
 static int fusezip_truncate(const char *path, off_t offset) {
@@ -260,22 +259,12 @@ static int fusezip_truncate(const char *path, off_t offset) {
         return res;
     }
     if ((res = node->truncate(offset)) != 0) {
-        return res;
+        node->close();
+        return -res;
     }
     return node->close();
 }
 
-int remove_node(FileNode *node) {
-    node->detach();
-    int id = node->id;
-    delete node;
-    if (id >= 0) {
-        return (zip_delete (get_zip(), id) == 0)? 0 : -ENOENT;
-    } else {
-        return 0;
-    }
-}
-
 static int fusezip_unlink(const char *path) {
     if (*path == '\0') {
         return -ENOENT;
@@ -287,7 +276,7 @@ static int fusezip_unlink(const char *path) {
     if (node->is_dir) {
         return -EISDIR;
     }
-    return remove_node(node);
+    return -get_data()->removeNode(node);
 }
 
 static int fusezip_rmdir(const char *path) {
@@ -304,7 +293,7 @@ static int fusezip_rmdir(const char *path) {
     if (!node->childs.empty()) {
         return -ENOTEMPTY;
     }
-    return remove_node(node);
+    return -get_data()->removeNode(node);
 }
 
 static int fusezip_mkdir(const char *path, mode_t mode) {
@@ -337,13 +326,18 @@ static int fusezip_rename(const char *path, const char *new_path) {
     }
     FileNode *new_node = get_file_node(new_path + 1);
     if (new_node != NULL) {
-        remove_node(new_node);
+        int res = get_data()->removeNode(new_node);
+        if (res !=0) {
+            return -res;
+        }
     }
 
     int len = strlen(new_path);
+    int oldLen = strlen(path + 1) + 1;
     char *new_name;
     if (!node->is_dir) {
-        len--;
+        --len;
+        --oldLen;
     }
     new_name = (char*)malloc(len + 1);
     if (new_path == NULL) {
@@ -367,21 +361,25 @@ static int fusezip_rename(const char *path, const char *new_path) {
                 for (nodelist_t::const_iterator i = n->childs.begin(); i != n->childs.end(); ++i) {
                     FileNode *nn = *i;
                     q.push(nn);
-                    char *name = (char*)malloc(len + strlen(nn->name) + 1);
+                    char *name = (char*)malloc(len + strlen(nn->full_name) - oldLen + (nn->is_dir ? 2 : 1));
                     if (name == NULL) {
                         //TODO: check that we are have enough memory before entering this loop
                         return -ENOMEM;
                     }
                     strcpy(name, new_name);
-                    strcpy(name + len, nn->name);
-                    nn->rename_wo_reparenting(name);
+                    strcpy(name + len, nn->full_name + oldLen);
+                    if (nn->is_dir) {
+                        strcat(name, "/");
+                    }
                     zip_rename(z, nn->id, name);
+                    nn->rename_wo_reparenting(name);
                 }
             }
         }
         zip_rename(z, node->id, new_name);
         // Must be called after loop because new_name will be truncated
         node->rename(new_name);
+
         return 0;
     }
     catch (...) {
@@ -616,11 +614,13 @@ int main(int argc, char *argv[]) {
     param.fileName = NULL;
 
     if (fuse_opt_parse(&args, &param, fusezip_opts, process_arg)) {
+        fuse_opt_free_args(&args);
         return EXIT_FAILURE;
     }
 
     // if all work is done inside options parsing...
     if (param.help) {
+        fuse_opt_free_args(&args);
         return EXIT_SUCCESS;
     }
     
@@ -629,10 +629,12 @@ int main(int argc, char *argv[]) {
         // no file name passed
         if (param.fileName == NULL) {
             print_usage();
+            fuse_opt_free_args(&args);
             return EXIT_FAILURE;
         }
 
         if ((data = initFuseZip(param.fileName)) == NULL) {
+            fuse_opt_free_args(&args);
             return EXIT_FAILURE;
         }
     }
@@ -680,7 +682,9 @@ int main(int argc, char *argv[]) {
     int res;
 
     fuse = fuse_setup(args.argc, args.argv, &fusezip_oper, sizeof(fusezip_oper), &mountpoint, &multithreaded, data);
+    fuse_opt_free_args(&args);
     if (fuse == NULL) {
+        delete data;
         return EXIT_FAILURE;
     }
     res = fuse_loop(fuse);
